	include "ioequ.asm"

DMA_SIZE	equ	2048
OUT_Buf		equ	8192
OUT_Buf_mod	equ	DMA_SIZE-1
DMA_IN_Buf	equ	OUT_Buf+DMA_SIZE
DMA_IN_mod	equ	DMA_SIZE-1
IN_Buf		equ	0
IN_Buf_mod	equ	250
A_PAR		equ	DMA_IN_Buf+DMA_IN_mod+1 ; address of filter para.

;; In the standard configuration, only 8Kwords are available, therefore
;; the maximum number of filter coefficients is 6144.
;; If you have more DSP memory, you can use filters with longer impulse 
;; responses. Just change the corresponding line in the C code.


DM_R_REQ	equ	$050001		;message -> host to request dma
DM_W_REQ	equ	$040002		;message to host to request dma-IN 
VEC_R_DONE	equ	$0024		;host command indicating dma complete
DMA_DONE	equ	$0		;DMA ended bit.
DMA_ACCEPTED 	equ	$1



;;;------------------------- Variable locations
;;;

x_sFlags	equ	$00ff		; dspstream flags
stereo		equ	$00fe		; stereo flag
sink		equ	$00fd
flipflop	equ	$00ff
save		equ	$00fe

cra_init 	equ	$4300           ;
crb_init 	equ	$2a00 		;
pcc     	equ	$1F7 	; both serial ports (SC0 not available)
pcddr	 	equ	$01c            ;
pcd_init 	equ	$0010           ;
	



;; INIT_INT initialise les adresses d'interruptions

INIT_I	MACRO

	org	p:$0			
	jmp	reset


	org	p:VEC_R_DONE
	bset	#DMA_DONE,x:x_sFlags
		
	org	p:$C			;SSI data received interrupt
	jsr	getssi
	nop

	org	p:$E			;SSI data received exception interrupt
	nop
	nop

	org	p:40			;Host Command interrupt
	jsr	go
	nop


	ENDM


; INPUT lit un echantillon et le place en x:(R5). INPUT utilise X0, Y0 et a!
;

INPUT	MACRO

	move	X1,y:save
_wait_in
	move	R6,a
	move	#>40,X0
	cmp	X0,a	#>$8000,X1	; a - X0 (R6 - 30)
	jlt	_wait_in

	move	x:(R1)+,X0		; get the sample
	mpyr	X0,X1,a	(R6)-		; Decrement R6 (one sample less).
	move	a,x:(R5)

	btst	#0,x:stereo
	jcc	_input_end		
	move	x:(R1)+,X0		; get the sample
	btst	#2,x:stereo		; if stereo
	jcs	_input_stereo
	macr	X0,X1,a (R6)-		; if sum add sample together.
	move	a,x:(R5)
	asr	a
	jmp	_input_end		
	
_input_stereo	
	mpyr	X0,X1,a (R6)-
	move	a,y:(R5)

_input_end
	move	(R5)+
	move	y:save,X1
	ENDM



	
; OUTPUT write a sample in the output buffer, and triggers a DMA if necessary
; Uses a,b and updates R3.
	
OUTPUT	MACRO
		
_output_next	
	move	a,x:(R3)+
	btst	#2,x:stereo
	jcc	<_output_test		
	move	b,x:(R3)+
	 
_output_test
	move	#>OUT_Buf,a
	move	R3,b
	cmp	a,b			; R3 - OUT_Buff
	jne	<_end_output		; si R3!= OUT_BUF, continuer...
	
send_DMA				; On a un buffer complet a envoyer.
	jclr	#m_htde,x:m_hsr,send_DMA
	movep	#DM_R_REQ,x:m_htx	; send "DSP_dm_R_REQ" to host
_ackBegin
	jclr	#m_hf1,x:m_hsr,_ackBegin	;    wait for HF1 to go high
	move	#>DMA_SIZE,b
	do	b,_prodDMA
	move	x:(R3)+,a		; and pass the values to the host.
_wait
	jclr	#m_htde,x:m_hsr,_wait
	move	a,x:m_htx	
_prodDMA
	btst	#DMA_DONE,x:x_sFlags
	jcs	_endDMA
	jclr	#m_htde,x:m_hsr,_prodDMA
	movep	#0,x:m_htx		;send zeros until noticed
	jmp	_prodDMA
_endDMA
	bclr	#DMA_DONE,x:x_sFlags	;reset the goddam flag!
_ackEnd

_end_output
	
	ENDM	



IRSMPL	MACRO

	move	#>IN_Buf,R2
		; R2 pointe vers les echantillons lus
	move	#>IN_Buf,R5
		; Et R5 aussi.
	move	#>IN_Buf,R7		
	move	#>IN_Buf_mod,M2
	move	#>IN_Buf_mod,M5
	move	#>DMA_IN_Buf,R1
	; R1 pointe vers le DMA IN
	move	#>DMA_IN_mod,M1
	move	#>DMA_IN_Buf,R0
	move	#>DMA_IN_mod,M0
	move	#>OUT_Buf,R3
	move	#>OUT_Buf_mod,M3
	move	#>0,R6
	clr 	a
	rep	#IN_Buf_mod+1		
	move	A,x:(R5)+
	rep	#IN_Buf_mod+1
	move	A,y:(R5)+
	rep	#OUT_Buf_mod+1
	move	A,x:(R3)+
	move	A,x:x_sFlags		;clear flags
	move	A,x:stereo		;clear flags
	move	A,y:flipflop		;clear flags
	ENDM

	
	
	
	
READPAR	MACRO
_wait_para0
	jclr	#m_hrdf,x:m_hsr,_wait_para0
	movep	x:m_hrx,x:stereo
	jsr 	sciset
	move	#>A_PAR,R4
	do	#4,_end_para1		; Lit P,Q,K-1 et KP
_wait_para1
	jclr	#m_hrdf,x:m_hsr,_wait_para1
	move	x:m_hrx,a
	move	a,x:(R4)+
_end_para1	
	do	a,_end_read		; Lit les KP coeff du filtre.
_wait_para2
	jclr	#m_hrdf,x:m_hsr,_wait_para2
	movep	x:m_hrx,x:(R4)+
_end_read
	ENDM


RSMPL	MACRO

	move		#>A_PAR,R4
	nop
	move		X:(R4)+,Y1			; Y1= P
	sub X0,B 	X:(R4)+,Y0			; Y0= Q
	move		X:(R4)+,X1			; X1= K-1
	move		R7,B				; 
	add Y0,B	X:(R4)+,X0
			; X0 = K.P , B = R7+Q
	tfr		Y1,B B1,Y0			; Y0 = R7+Q  B = P
_iteration
	cmp Y0,B					; Si R7+Q < i*P continue
	jgt _start
	
	INPUT						; attention: (R5)+
							; R5= pointeur d'ecriture
	add Y1,B
	jmp _iteration
	
_start	
	sub Y1,B					
	sub Y0,B
	neg B
	move B,R7
	move		R7,N4
	move		R5,R2				; set pointeur interne R2.
	move		(R4)+N4				; set pointeur debut coeffs
	move		Y1,N4				; N4= P
	btst #2,x:stereo
	jcc _mono					; enlever le ;	
	clr B		Y:(R2),Y1
	clr A		X:(R2)-,X0	Y:(R4)+N4,Y0	; Y0= H(0).
	do X1,_end_loop					; Pour i=0..K-2
	 mac Y1,Y0,B	Y:(R2),Y1	
	 mac X0,Y0,A	X:(R2)-,X0	Y:(R4)+N4,Y0	; { A= A + X(n-i)*H(i)}
_end_loop						
	macr X0,Y0,A					; Idem i=K-1, arrondi sortie.
	macr Y1,Y0,B					; Idem i=K-1, arrondi sortie.
	
	jmp _end_rsmpl
_mono
	clr A		X:(R2)-,X0	Y:(R4)+N4,Y0	; Y0= H(0).
	do X1,_end_loop_mono				; Pour i=0..K-2
	 mac X0,Y0,A	X:(R2)-,X0	Y:(R4)+N4,Y0	; { A= A + X(n-i)*H(i)
_end_loop_mono						; do loop necess. for interr.
	macr X0,Y0,A					; Idem i=K-1, arrondi sortie.
	move	A,B
	
_end_rsmpl
	OUTPUT						; sortie de A
(et B)
 
	ENDM

