	include "ioequ.asm"

DMA_SIZE	equ	2048
OUT_Buf		equ	8192
OUT_Buf_mod	equ	DMA_SIZE-1
DMA_IN_Buf	equ	OUT_Buf+DMA_SIZE
DMA_IN_mod	equ	DMA_SIZE-1
IN_Buf		equ	0
IN_Buf_mod	equ	250
A_PAR		equ	DMA_IN_Buf+DMA_IN_mod+1 ; address of filter para.

;; In the standard configuration, only 8Kwords are available, therefore
;; the maximum number of filter coefficients is 6144.
;; If you have more DSP memory, you can use filters with longer impulse 
;; responses. Just change the corresponding line in the C code.


DM_R_REQ	equ	$050001		;message -> host to request dma
DM_W_REQ	equ	$040002		;message to host to request dma-IN 
VEC_R_DONE	equ	$0024		;host command indicating dma complete
DMA_DONE	equ	$0		;DMA ended bit.
DMA_ACCEPTED 	equ	$1



;;;------------------------- Variable locations
;;;

x_sFlags	equ	$00ff		; dspstream flags
stereo		equ	$00fe		; stereo flag

	



;; INIT_INT initialise les adresses d'interruptions

INIT_I	MACRO

	org	p:$0			
	jmp	reset


	org	p:VEC_R_DONE
	bset	#DMA_DONE,x:x_sFlags
		
	org	p:$2C			; DMA-IN accepted: start reading.
	jsr	startDMA_In		

	ENDM


; INPUT lit un echantillon et le place en x:(R5). INPUT utilise X0 et a!
;

INPUT	MACRO

	move	R1,a
	move	#>DMA_IN_Buf,X0
	cmp	X0,a
	jseq	call_dma_in
	btst	#0,x:stereo
	jcc	_input_wait	
	
_input_wait0
	move	x:(R1)+,X0		; get the sample
	move	X0,a
	jclr	#15,X0,_ooo0		; This is a modification which corrects
	move	#>$FF,a2		; the driver's "bug". It sign-extends
	move	#>$FF0000,X0		; the received short value,if necessary
	or	X0,a	
_ooo0					; now the correct value is in a.
	move	a,y:(R5)
	
_input_wait
	move	x:(R1)+,X0		; get the sample
	move	X0,a
	jclr	#15,X0,_ooo		; This is a modification which corrects
	move	#>$FF,a2		; the driver's "bug". It sign-extends
	move	#>$FF0000,X0		; the received short value,if necessary
	or	X0,a	
_ooo					; now the correct value is in a.
	move	a,x:(R5)+
	
	ENDM



DMA_IN	MACRO

call_dma_in
	jset	#m_hf1,x:m_hsr,call_dma_in

_wait00
	jclr	#m_htde,x:m_hsr,_wait00
	movep	#>DM_W_REQ,x:m_htx
_ready00
	btst 	#DMA_ACCEPTED,x:x_sFlags
	jcc	_ready00

	move	#DMA_SIZE,a
	do	a,_end_DMA_loop
_clear
	jclr	#m_hrdf,x:m_hsr,_clear	
	movep	x:m_hrx,x:(R1)+
_end_DMA_loop
	jclr	#m_hrdf,x:m_hsr,_then	
	move	x:m_hrx,X0		; Continue reading incoming data...
_then
	jset	#m_hf1,x:m_hsr,_end_DMA_loop	; until HF1 is reset.
	rts
	
		
startDMA_In
	jclr	#m_hrdf,x:m_hsr,startDMA_In	
	movep	x:m_hrx,X0
	bset	#DMA_ACCEPTED,x:x_sFlags	; But we don't really need it.
	rti

	ENDM

	
	
; OUTPUT write a sample in the output buffer, and triggers a DMA if necessary
; Uses a,b and updates R3.
	
OUTPUT	MACRO
		
	move	a,x:(R3)+
	btst	#1,x:stereo
	jcs	<_put_b	
	btst	#0,x:stereo
	jcc	<_output_test
_put_b
	move	b,x:(R3)+
	 
_output_test
	move	#>OUT_Buf,a
	move	R3,b
	cmp	a,b			; R3 - OUT_Buff
	jne	<_end_output		; si R3!= OUT_BUF, continuer...
	
send_DMA				; On a un buffer complet a envoyer.
	jclr	#m_htde,x:m_hsr,send_DMA
	movep	#DM_R_REQ,x:m_htx	; send "DSP_dm_R_REQ" to host
_ackBegin
	jclr	#m_hf1,x:m_hsr,_ackBegin	;    wait for HF1 to go high
	move	#>DMA_SIZE,b
	do	b,_prodDMA
	move	x:(R3)+,a		; and pass the values to the host.
_wait
	jclr	#m_htde,x:m_hsr,_wait
	move	a,x:m_htx	
_prodDMA
	btst	#DMA_DONE,x:x_sFlags
	jcs	_endDMA
	jclr	#m_htde,x:m_hsr,_prodDMA
	movep	#0,x:m_htx		;send zeros until noticed
	jmp	_prodDMA
_endDMA
	bclr	#DMA_DONE,x:x_sFlags	;reset the goddam flag!
_ackEnd

_end_output
	
	ENDM	



IRSMPL	MACRO

	move	#>IN_Buf,R2
		; R2 pointe vers les echantillons lus
	move	#>IN_Buf,R5
		; Et R5 aussi.
	move	#>IN_Buf,R7		
	move	#>IN_Buf_mod,M2
	move	#>IN_Buf_mod,M5
	move	#>DMA_IN_Buf,R1
	; R1 pointe vers le DMA IN
	move	#>DMA_IN_mod,M1
	clr 	a
	rep	#IN_Buf_mod+1		
	move	A,x:(R5)+
	rep	#IN_Buf_mod+1
	move	A,y:(R5)+
	move	#>OUT_Buf,R3
	move	#>OUT_Buf_mod,M3
	move	R2,x:x_sFlags		;clear flags
	rep	#OUT_Buf_mod+1
	move	A,x:(R3)+
	bclr	#0,x:stereo			
	bclr	#1,x:stereo
	move	#>1,A			; To force the driver to create a ramp!!!
	move	a,x:(R3)+
	move	a,x:(R3)+
	ENDM

;; Forcing the driver to create a ramp is necessary (for a reason I don't quite get!)
;; If you send null initial samples, no ramp is created and the driver often hangs...
;; So don't try to understand, and send a non-zero first pair of samples!!!
	
	
	
	
READPAR	MACRO
_wait_para0
	jclr	#m_hrdf,x:m_hsr,_wait_para0
	movep	x:m_hrx,x:stereo
	move	#>A_PAR,R4
	do	#4,_end_para1		; Lit P,Q,K-1 et KP
_wait_para1
	jclr	#m_hrdf,x:m_hsr,_wait_para1
	move	x:m_hrx,a
	move	a,x:(R4)+
_end_para1	
	do	a,_end_read		; Lit les KP coeff du filtre.
_wait_para2
	jclr	#m_hrdf,x:m_hsr,_wait_para2
	movep	x:m_hrx,x:(R4)+
_end_read
	ENDM


RSMPL	MACRO

	move		#>A_PAR,R4
	nop
	move		X:(R4)+,Y1			; Y1= P
	sub X0,B 	X:(R4)+,Y0			; Y0= Q
	move		X:(R4)+,X1			; X1= K-1
	move		R7,B				; 
	add Y0,B	X:(R4)+,X0
			; X0 = K.P , B = R7+Q
	tfr		Y1,B B1,Y0			; Y0 = R7+Q  B = P
_iteration
	cmp Y0,B					; Si R7+Q < i*P continue
	jgt _start
	
	INPUT						; attention: (R5)+
							; R5= pointeur d'ecriture
	add Y1,B
	jmp _iteration
	
_start	
	sub Y1,B					
	sub Y0,B
	neg B
	move B,R7
	move		R7,N4
	move		R5,R2				; set pointeur interne R2.
	move		(R4)+N4				; set pointeur debut coeffs
	move		Y1,N4				; N4= P
	btst #0,x:stereo
	jcc _mono	
	clr B		Y:(R2),Y1
	clr A		X:(R2)-,X0	Y:(R4)+N4,Y0	; Y0= H(0).
	do X1,_end_loop					; Pour i=0..K-2
	 mac Y1,Y0,B	Y:(R2),Y1	
	 mac X0,Y0,A	X:(R2)-,X0	Y:(R4)+N4,Y0	; { A= A + X(n-i)*H(i)}
_end_loop						
	macr X0,Y0,A					; Idem i=K-1, arrondi sortie.
	macr Y1,Y0,B					; Idem i=K-1, arrondi sortie.
	
	jmp _end_rsmpl
_mono
	clr A		X:(R2)-,X0	Y:(R4)+N4,Y0	; Y0= H(0).
	rep X1						; Pour i=0..K-2
	 mac X0,Y0,A	X:(R2)-,X0	Y:(R4)+N4,Y0	; { A= A + X(n-i)*H(i)
	macr X0,Y0,A					; Idem i=K-1, arrondi sortie.
	move	A,B
	
_end_rsmpl
	OUTPUT						; sortie de A
(et B)
 
	ENDM

