;;; EXAMPLE 2 - adding harmonics to build a wavetable

; build-harmonic uses s-osc to resample a sine to get the nth harmonic
;
(defun build-harmonic (n)
  (let ((fund (/ 22050.0 2048.0)))
    (s-osc (car *osc*) (hz-to-step fund)
	   22050.0 (* n fund)
	   (/ 1.0 fund)
	   0.0 0)))

; mkwave uses build-harmonic to build and add several harmonics together
;
(defun mkwave ()
  (setf *WAVE* (s-add (s-scale (build-harmonic 1.0) 1.0)
		      (s-add (s-scale (build-harmonic 2.0) 0.5)
			     (s-add (s-scale (build-harmonic 3.0) 0.25)
				    (s-scale (build-harmonic 4.0) 0.12)))))
  (setf *WAVE* (cons *WAVE* (cdr *OSC*))))


; this expression runs mkwave if *wave* is undefined (e.g. the first time
;   you load this file).
;
(if (not (boundp '*wave*)) (mkwave))

; note uses *wave* to produce a tone with several harmonics
;
(defun note (pitch dur) (osc pitch dur 0 *wave*))

; now try it out
;
(play (seq (note c4 i) 
	   (note d4 i) 
	   (note f4 i) 
	   (note g4 i)
	   (note d4 q)))
