;;; EXAMPLE 7 - finding zero crossings, s-samples

; if demo4.lsp was not loaded, load sound sample:
(if (not (boundp 'a-snd)) 
    (setf a-snd (sf-load "/usr0/rbd/fugue/test/ex/demo-snd.nh" 22050.0)))


(defun zeros (snd)
  ; start by getting the samples, only take 1000 samples max
  (prog ((s (s-samples snd 1000))
	 newsign sign n len result result2 starttime srate)
    ; go through the array looking for zero crossings
    (setf len (length s))
    ; stop if there are no samples
    (if (= len 0) (return nil))
    (setf sign (> 0.0 (aref s 0)))
    ; get the start time and sample rate of the sound for use below
    (setf starttime (car (s-extent snd)))
    (setf srate (s-srate snd))
    (setf n 1)
   loop
    (if (>= n len) (go done))
    (setf newsign (> 0.0 (aref s n)))
    (if (not (eq sign newsign)) (setf result (cons n result)))
    (setf sign newsign)
    (setf n (1+ n))
    (go loop)
   done ; now we have the zero crossings, convert them to times
    (dolist (num result)
      ; return the time of the zero crossing, which is the start time
	    of the snd plus the sample number / srate

      (setf result2 (cons (+ starttime (/ num srate))
			  result2)))
		       
    (return result2)))


; extract a short piece of this sample
;
(setf short (extract 0.1 0.14 (cue a-snd)))

(setf z (zeros short))
(print z)


; find the differences between zero crossings reported by zeros
; print the result in terms of samples for readability
;
(defun periods (lis short)
  (prog (result prev srate)
    (if (null lis) (return nil))
    (setf srate (s-srate short))
   loop
    (setf prev (car lis))
    (setf lis (cdr lis))
    (if (null lis) (return (reverse result)))
    (setf result (cons (* srate (- (car lis) prev)) result))
    (go loop)))

(setf p (periods z short))
(format t "The intervals (in samples) between zero crossings are: ~%~A~%" p)


; build a wavetable using zero crossing information
;
; I interactively played with the data and decided to extract from the
; 5th period to the 21st period (these had 86 and 87 samples each and
; seem to indicate some kind of periodicity).  The 1st period measures
; from the zeroth zero crossing to the first, so the 5th period measures
; from the 4th zero crossing to the 5th.  I'll arbitrarily take
; the 4th and 20th zero crossing times (the 5th and 20th should work as
; well), and from the data, this looks like 2 waveform periods.
;
; arguments are:
;  snd - the sound to extract from
;  zeros - the result of (zeros snd)
;  start - the number of the starting zero crossing
;  stop - the number of the ending zero crossing
;  n - number of periods contained in the extracted sound
;
(defun extract-table (snd zeros start stop n)
  (let (starttime extent hz)
    ; we're going to do this the hard way because there seems to be a
    ; bug in extract or s-apply.  So, start by shifting snd to time zero:
    (setf starttime (car (s-extent snd)))
    (setf snd (at (- starttime) (cue snd)))

    (format t "~A~%" snd)
    ; also get the start and stop times and shift them:
    (setf start (- (nth start zeros) starttime))
    (setf stop (- (nth stop zeros) starttime))

    (format t "table ~A start ~A stop ~A~%" snd start stop)

    ; now extract the samples of interest, note that we are
    ; overwriting our pointer to the snd argument
    (setf snd (extract start stop (cue snd)))
    (format t "table now ~A~%" snd)

    ; now we need to figure out the pitch this sound would represent
    ; when played at its samplerate.  The pitch in hz is 1 / duration,
    ; and duration is the extent of the sound / n.  Therefore, take
    ; n/extent
    (setf extent (s-extent snd))
    (setf hz (/ n (- (cadr extent) (car extent))))
    ; an osc table is the sound cons'd with the pitch number
    (cons snd (hz-to-step hz))))


; try it out
;
(setf *a-voice* (extract-table short z 4 20 2))

; now use the table with an oscillator
;
(play (osc c3 1.0 0 *a-voice* ))

; play it at its normal pitch
;
(play (osc (cdr *a-voice*) 1.0 0 *a-voice*))