;;; EXAMPLE 8 - glissandi, fmosc


; make a short sine tone with no frequency modulation
;
(play (fmosc c4 (pwl 0.1)))

; make a longer sine tone -- note that the duration of the modulator
; determines the duration of the tone
;
(play (fmosc c4 (pwl 0.5)))

; make a frequency sweep of one octave; the piece-wise linear function
; sweeps from 0 to (step-to-hz c4) because, when added to the c4
; fundamental, this will double the frequency and cause an octave sweep.
;
(play (fmosc c4 (pwl 0.5 (step-to-hz c4) 0.501)))

; do the same thing with a non-sine table
;
;   first, make sure our standard sound is loaded
;
; if demo4.lsp was not loaded, load sound sample:
(if (not (boundp 'a-snd)) 
    (setf a-snd (sf-load "/usr0/rbd/fugue/test/ex/demo-snd.nh" 22050.0)))
;
; using data from demo7, extract a table (same as *a-voice* in demo7)
;
(setf *fm-voice* (cons (extract 0.110204 0.13932 (cue a-snd)) 24.848422))

(play (fmosc cs2 (pwl 0.5 (step-to-hz cs2) 0.501) 0 *fm-voice* 0.0))

; try some vibrato, use LFO to make low freq sine modulation.  The
; generated sine tone has an amplitude of 32768, so scale down to 10 hz.
;
(play (fmosc cs2 (s-scale (lfo 6.0) (/ 10.0 32768.0)) 0 *fm-voice* 0.0))


; make a function to give a frequency sweep, starting after <delay>
; seconds, then sweeping from <pitch-1> to <pitch-2> in <sweep-time>
; seconds and then holding at <pitch-2> for <hold-time> seconds.
;
(defun sweep (delay pitch-1 sweep-time pitch-2 hold-time)
  (let ((interval (- (step-to-hz pitch-2)
		     (step-to-hz pitch-1))))
    (pwl delay 0.0 
	 ; sweep from pitch 1 to pitch 2
	 (+ delay sweep-time) interval
	 ; hold until about 1 sample from the end
	 (+ delay sweep-time hold-time -0.0005) interval
	 ; quickly ramp to zero (pwl always does this, so make it short)
	 (+ delay sweep-time hold-time))))


; now try it out
;
(play (fmosc cs2 (sweep 0.1 cs2 0.6 gs2 0.5) 0 *fm-voice* 0.0))


; make the obligatory fm sound
;
(setf modulator (s-mult (pwl 1.0 1.0 1.0005)
			(osc c4)))

; scale to reduce modulation to 1000 hz
(setf modulator (s-scale modulator (/ 1000.0 32768.0)))

; make the sound
(play (fmosc c4 modulator))
